package com.steerinfo.dil.controller;

import com.alibaba.fastjson.JSON;
import com.steerinfo.dil.feign.ESFeign;
import com.steerinfo.dil.model.AmsshipDeliveryAttorney;
import com.steerinfo.dil.util.BaseRESTfulController;
import com.steerinfo.dil.util.ColumnDataUtil;
import com.steerinfo.dil.util.PageListAdd;

import com.steerinfo.framework.controller.RESTfulResult;
import com.steerinfo.framework.service.pagehelper.PageHelper;
import com.steerinfo.framework.service.pagehelper.PageList;
import com.steerinfo.framework.utils.collection.ListUtils;
import com.steerinfo.dil.model.OmsshipShipmentInstructions;
import com.steerinfo.dil.service.IOmsshipShipmentInstructionsService;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;
import java.math.BigDecimal;

/**
 * OmsshipShipmentInstructions RESTful接口：
 * @author generator
 * @version 1.0-SNAPSHORT 2021-08-20 03:05
 * 类描述
 * 修订历史：
 * 日期：2021-08-20
 * 作者：generator
 * 参考：
 * 描述：OmsshipShipmentInstructions RESTful接口
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@RestController
@RequestMapping("/${api.version}/omsshipshipmentinstructions")
public class OmsshipShipmentInstructionsController extends BaseRESTfulController {

    @Autowired
    IOmsshipShipmentInstructionsService omsshipShipmentInstructionsService;
    @Autowired
    ColumnDataUtil columnDataUtil;
    @Autowired
    ESFeign esFeign;

    /**
     * 新增
     * @param map
     * @return
     */
    @ApiOperation(value="新增", notes="根据OmsshipShipmentInstructions对象创建")
    @ApiImplicitParam(name = "omsshipShipmentInstructions", value = "map", required = true, dataType = "map", paramType = "java.util.Map")
    @PostMapping(value = "/addShipmentInstructions")
    public RESTfulResult addShipmentInstructions(@RequestBody Map<String,Object> map){
        int code=omsshipShipmentInstructionsService.insert(map);
        if (code!=1){
            return failed("添加失败，请仔细核实信息是否正确");
        }
        return success(code);
    }

    /**
     * 查询
     * @param instructionsId
     * @return
     */
    @ApiOperation(value="渲染信息", notes="根据url的id来获取详细信息")
    @ApiImplicitParam(paramType = "path", name = "instructionsId", value = "ID", required = true, dataType = "BigDecimal")
    @PostMapping(value = "/getShipmentInstructions/{instructionsId}")
    public RESTfulResult getShipmentInstructions(@PathVariable("instructionsId") BigDecimal instructionsId){
        List<Map<String,Object>> map = omsshipShipmentInstructionsService.selectShipmentInstructions(instructionsId);
        return success(map);
    }

    /**
     * 修改
     * @param omsshipShipmentInstructions
     * @return
     */
    @ApiOperation(value="更新详细信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "omsshipShipmentInstructions", value = "详细实体omsshipShipmentInstructions", required = true, dataType = "OmsshipShipmentInstructions")
    })
    @PostMapping(value = "/updateShipmentInstructions")
    public RESTfulResult updateShipmentInstructions(@RequestBody Map<String,Object> omsshipShipmentInstructions){
        int code = omsshipShipmentInstructionsService.updateByPrimaryKey(omsshipShipmentInstructions);
        return success(code);
    }

    /**
     * 删除
     * @param instructionsId
     * @return
     */
    @ApiOperation(value="删除", notes="根据url的id来指定删除对象")
    @ApiImplicitParam(paramType = "BigDecimal", name = "instructionsId", value = "ID", required = true, dataType = "BigDecimal")
    @PostMapping(value = "/deleteByPrimaryKey/{instructionsId}")
    public RESTfulResult deleteByPrimaryKey(@PathVariable("instructionsId") BigDecimal instructionsId){
        omsshipShipmentInstructionsService.deleteByPrimaryKey(instructionsId);
        return success();
    }

    /**
     * 展示所有装船指令
     * @param mapVal
     * @param pageNum
     * @param pageSize
     * @param apiId
     * @return
     */
    @ApiOperation(value="展示装船指令表", notes="分页查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "apiId", value = "74", required = false, dataType = "BigDecimal"),
    })
    @PostMapping(value = "/getShipMentInstructionsList")
    public RESTfulResult getShipMentInstructionsList(@RequestBody(required = false) Map<String,Object> mapVal,
                                                     Integer pageNum,
                                                     Integer pageSize,
                                                     Integer apiId,
                                                     String status,
                                                     String con){
        //判断状态值是否为空
        if (status!=null){
            mapVal.put("status",status);
        }
        if (con!=null && !"null".equals(con)){
            mapVal.put("con","%" + con + "%");
        }
        List<Map<String, Object>> detailListTotal =  omsshipShipmentInstructionsService.selectAll(mapVal);
        PageHelper.startPage(pageNum, pageSize);
        //分页查询数据
        List<Map<String, Object>> columnList = omsshipShipmentInstructionsService.selectAll(mapVal);
        PageListAdd data = columnDataUtil.tableColumnData(apiId, detailListTotal, columnList);
        return success(data);

    }


    //下发
    @ApiOperation(value="获取详细信息", notes="根据url的id来获取详细信息")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "BigDecimal")
    @PostMapping(value = "/sendShipmentInstructionsStatus/{shipmentInstructionsId}")
    public RESTfulResult sendShipmentInstructionsStatus(@PathVariable("shipmentInstructionsId") Integer shipmentInstructionsId){
        OmsshipShipmentInstructions omsshipShipmentInstructions=new OmsshipShipmentInstructions();
        omsshipShipmentInstructions.setShipmentInstructionsId(new BigDecimal(shipmentInstructionsId));
        String status="已下发";
        omsshipShipmentInstructions.setInstructionIssueStatus(status);
        omsshipShipmentInstructionsService.updateByPrimaryKeySelective(omsshipShipmentInstructions);
        return success(omsshipShipmentInstructions);
    }
    /**
     * 获取批次id
     * @return
     */
    @GetMapping("getBatchId")
    @ApiOperation(value = "获取批次id下拉列表")
    public RESTfulResult getBatchId(){
        return success(omsshipShipmentInstructionsService.getBatchId());
    }

}
