package com.steerinfo.dil.controller;

import com.alibaba.fastjson.JSON;
import com.steerinfo.dil.feign.ESFeign;
import com.steerinfo.dil.model.TmsshipLoadShipResult;
import com.steerinfo.dil.model.TmsshipWaterQualityResult;
import com.steerinfo.dil.service.ITmsshipWaterQualityResultService;
import com.steerinfo.dil.util.BaseRESTfulController;
import com.steerinfo.dil.util.ColumnDataUtil;
import com.steerinfo.dil.util.PageListAdd;
import com.steerinfo.framework.controller.RESTfulResult;
import com.steerinfo.framework.service.pagehelper.PageHelper;
import com.steerinfo.framework.service.pagehelper.PageList;
import com.steerinfo.framework.utils.collection.ListUtils;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;
import java.math.BigDecimal;

/**
 * TmsshipWaterQualityResult RESTful接口：
 * @author generator
 * @version 1.0-SNAPSHORT 2021-08-19 09:01
 * 类描述
 * 修订历史：
 * 日期：2021-08-19
 * 作者：generator
 * 参考：
 * 描述：TmsshipWaterQualityResult RESTful接口
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@RestController
@RequestMapping("/${api.version}/tmsshipwaterqualityresults")
public class TmsshipWaterQualityResultController extends BaseRESTfulController {

    @Autowired
    ITmsshipWaterQualityResultService tmsshipWaterQualityResultService;
    @Autowired
    ColumnDataUtil columnDataUtil;
    @Autowired
    ESFeign esFeign;
    /**
     * 新增水分质检作业
     * @param map
     * @return
     */
    @ApiOperation(value="新增水分质检作业", notes="根据TmsshipWaterQualityResult对象创建")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "map",value = "水分质检字段",required = true,paramType = "java.util.Map")
    })
    @PostMapping("/addWaterQualityResult")
    public RESTfulResult addWaterQualityResult(@RequestBody Map<String,Object> map) {
        int code = tmsshipWaterQualityResultService.insertWaterQualityResult(map);
        return success(code);
    }


    /**
     * 修改水分质检作业信息
     * @param tmsshipWaterQualityResult
     * @return
     */
    @ApiOperation(value="修改水分质检信息", notes="根据tmsshipWaterQualityResult对象修改")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "tmsshipWaterQualityResult",value = "水分质检字段",required = true,paramType = "TmsshipLoadShipResult")
    })
    @PostMapping("/updateWaterQualityResult")
    public RESTfulResult updateWaterQualityResult(@RequestBody TmsshipWaterQualityResult tmsshipWaterQualityResult) {
        int code = tmsshipWaterQualityResultService.updateWaterQualityResult(tmsshipWaterQualityResult);
        return success(code);
    }

    /**
     * 删除水分质检信息
     * @param resultId
     * @return
     */
    @ApiOperation(value="删除水分质检信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "resultId",value = "水分质检字段",required = true,paramType = "BigDecimal")
    })
    @PostMapping("/deleteWaterQualityResult/{resultId}")
    public RESTfulResult deleteWaterQualityResult(@PathVariable("resultId") BigDecimal resultId) {
        int code = tmsshipWaterQualityResultService.deleteWaterQualityResult(resultId);
        return success(code);
    }

    /**
     * 查询水分质检信息
     * @param reusltId
     * @return
     */
    @ApiOperation(value="查询水分质检信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "reusltId",value = "水分质检字段",required = true,paramType = "java.util.Map")
    })
    @PostMapping("/selectWaterQualityResult/{reusltId}")
    public RESTfulResult selectWaterQualityResult(@PathVariable("reusltId") BigDecimal reusltId) {
        TmsshipWaterQualityResult tmsshipWaterQualityResult = tmsshipWaterQualityResultService.selectWaterQualityResult(reusltId);
        return success(tmsshipWaterQualityResult);
    }

    /**
     * 查询水分质检信息
     * @param resultId
     * @return
     */
    @ApiOperation(value="查询水分质检信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "resultId",value = "水分质检字段",required = true,paramType = "BigDecimal")
    })
    @PostMapping("/getWaterQuality/{resultId}")
    public RESTfulResult getWaterQuality(@PathVariable("resultId") BigDecimal resultId) {
        List<Map<String,Object>> tmsshipWaterQualityResult = tmsshipWaterQualityResultService.getWaterQuality(resultId);
        return success(tmsshipWaterQualityResult);
    }

    /**
     * 展示水分质检信息列表
     * @param mapVal
     * @param pageNum
     * @param pageSize
     * @param apiId
     * @return
     */
    @ApiOperation(value="展示水分质检信息列表", notes="分页查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "apiId", value = "67", required = false, dataType = "BigDecimal"),
    })
    @PostMapping(value = "/getWaterQualityResultList")
    public RESTfulResult getWaterQualityResultList(@RequestBody(required = false) Map<String,Object> mapVal,
                                                   Integer pageNum,
                                                   Integer pageSize,
                                                   Integer apiId,
                                                   String con){
        if (con != null && !"null".equals(con)) {
            mapVal.put("con","%" + con + "%");
        }
        List<Map<String, Object>> detailListTotal = tmsshipWaterQualityResultService.selectWaterQualityList(mapVal);
        PageHelper.startPage(pageNum, pageSize);
        //分页查询数据
        List<Map<String, Object>> columnList = tmsshipWaterQualityResultService.selectWaterQualityList(mapVal);
        PageListAdd data = columnDataUtil.tableColumnData(apiId, detailListTotal, columnList);
        return success(data);

    }
//根据选中的批次id获取合同水分值、实装吨位
@ApiOperation(value="根据选中的批次id获取合同水分值、实装吨位")
@ApiImplicitParams({
        @ApiImplicitParam(name = "batchId",value = "水分质检字段",required = true,paramType = "BigDecimal")
})
@PostMapping("/getMoistureAndInstallations")
public RESTfulResult getMoistureAndInstallations(@RequestBody(required = false) Map<String,Object> map) {
        Map<String,Object> result = tmsshipWaterQualityResultService.getMoistureAndInstallations(map);
        if (result.get("errCode")=="001"){
            return failed("请勾选船舶信息");
        }
       return success(result);

}

}
